@@ -1,17 +1,19 @@
 /* main.c
  *
- * Copyright (c) 1992-2000 by Mike Gleason.
+ * Copyright (c) 1992-2005 by Mike Gleason.
  * All rights reserved.
  * 
  */
 
 #include "syshdrs.h"
+#ifdef PRAGMA_HDRSTOP
+#	pragma hdrstop
+#endif
 #include "ls.h"
 #include "bookmark.h"
 #include "shell.h"
 #include "cmds.h"
 #include "main.h"
-#include "getopt.h"
 #include "progress.h"
 #include "pref.h"
 #include "readln.h"
@@ -20,35 +22,26 @@
 #include "spool.h"
 #include "util.h"
 
-#if defined(WIN32) || defined(_WINDOWS)
-	WSADATA wsaData;
-	int wsaInit = 0;
-
-	__inline void DisposeWinsock(int aUNUSED) { if (wsaInit > 0) WSACleanup(); wsaInit--; }
-#else
-#	define DisposeWinsock(a)
-#endif
-
 int gStartupUrlParameterGiven = 0;
 int gIsTTY, gIsTTYr;
 int gScreenColumns;
 
 FTPLibraryInfo gLib;
 FTPConnectionInfo gConn;
-LineList gStartupURLCdList;
+FTPLineList gStartupURLCdList;
+char *gXBuf = NULL;
+size_t gXBufSize = 0;
 int gTransferTypeInitialized = 0;
 int gTransferType;
 int gURLMode = 0;
 extern int gUnprocessedJobs;
 char gLocalCWD[512], gPrevLocalCWD[512];
 
-extern char *gOptArg;
-extern int gOptInd;
 extern char gRemoteCWD[512], gPrevRemoteCWD[512];
 extern Bookmark gBm;
 extern int gLoadedBm;
-extern int gFirstTimeUser;
 extern int gFirewallType;
+extern char gAutoAscii[];
 extern char gFirewallHost[64];
 extern char gFirewallUser[32];
 extern char gFirewallPass[32];
@@ -58,25 +51,37 @@ extern unsigned int gFirewallPort;
 extern int gConnTimeout, gXferTimeout, gCtrlTimeout;
 extern int gDataPortMode, gRedialDelay;
 extern int gDebug;
-extern int gNumProgramRuns;
+extern int gNumProgramRuns, gDoNotDisplayAds;
 extern int gSOBufsize;
 extern FTPProgressMeterProc gProgressMeter;
+extern char gOurHostName[64];
+extern int gGetOurHostNameResult;
 
 static void
 Usage(void)
 {
+	FILE *fp;
 #ifdef UNAME
 	char s[80];
 #endif
 
-	(void) fprintf(stderr, "\nUsage:  ncftp [-u user] [<host> | <URL>]\n");
-	(void) fprintf(stderr, "\nProgram version:  %s\nLibrary version:  %s\n", gVersion + 5, gLibNcFTPVersion + 5);
+	fp = stderr;
+	(void) fprintf(fp, "\nUsage:  ncftp [flags] [<host> | <directory URL to browse>]\n");
+	(void) fprintf(fp, "\nFlags:\n\
+  -u XX  Use username XX instead of anonymous.\n\
+  -p XX  Use password XX with the username.\n\
+  -P XX  Use port number XX instead of the default FTP service port (21).\n\
+  -j XX  Use account XX with the username (rarely needed).\n\
+  -F     Dump a sample $HOME/.ncftp/firewall prefs file to stdout and exit.\n");
+
+	(void) fprintf(fp, "\nProgram version:  %s\nLibrary version:  %s\n", gVersion + 5, gLibNcFTPVersion + 5);
 #ifdef UNAME
 	AbbrevStr(s, UNAME, 60, 1);
-	(void) fprintf(stderr, "System:           %s\n", s);
+	(void) fprintf(fp, "System:           %s\n", s);
 #endif
-	(void) fprintf(stderr, "\nThis is a freeware program by Mike Gleason (ncftp@ncftp.com).\n");
-	(void) fprintf(stderr, "Use ncftpget and ncftpput for command-line FTP.\n\n");
+	(void) fprintf(fp, "\nThis is a freeware program by Mike Gleason (http://www.NcFTP.com).\n");
+	(void) fprintf(fp, "A directory URL ends in a slash, i.e. ftp://ftp.freebsd.org/pub/FreeBSD/\n");
+	(void) fprintf(fp, "Use ncftpget and ncftpput for command-line FTP and file URLs.\n\n");
 	exit(2);
 }	/* Usage */
 
@@ -100,7 +105,7 @@ InitConnectionInfo(void)
 {
 	int result;
 
-	result = FTPInitConnectionInfo(&gLib, &gConn, kDefaultFTPBufSize);
+	result = FTPInitConnectionInfo2(&gLib, &gConn, gXBuf, gXBufSize);
 	if (result < 0) {
 		(void) fprintf(stderr, "ncftp: init connection info error %d (%s).\n", result, FTPStrError(result));
 		exit(1);
@@ -126,6 +131,11 @@ InitConnectionInfo(void)
 	gConn.dataSocketRBufSize = gConn.dataSocketSBufSize = gSOBufsize;
 	if (gRedialDelay >= 10)
 		gConn.redialDelay = gRedialDelay;
+	if ((gAutoAscii[0] == '\0') || (ISTREQ(gAutoAscii, "no")) || (ISTREQ(gAutoAscii, "off")) || (ISTREQ(gAutoAscii, "false"))) {
+		gConn.asciiFilenameExtensions = NULL;
+	} else {
+		gConn.asciiFilenameExtensions = gAutoAscii;
+	}
 }	/* InitConnectionInfo */
 
 
@@ -180,7 +190,7 @@ SetStartupURL(const char *const urlgiven)
 			gConn.hasSIZE = gBm.hasSIZE;
 			gConn.hasMDTM = gBm.hasMDTM;
 			gConn.hasPASV = gBm.hasPASV;
-			gConn.hasUTIME = gBm.hasUTIME;
+			gConn.hasSITE_UTIME = gBm.hasUTIME;
 			gConn.port = gBm.port;
 		} else {
 			SetBookmarkDefaults(&gBm);
@@ -217,7 +227,7 @@ SetStartupURL(const char *const urlgiven)
 static void
 OpenURL(void)
 {
-	LinePtr lp;
+	FTPLinePtr lp;
 	int result;
 
 	if (gURLMode == 1) {
@@ -253,20 +263,16 @@ PreInit(void)
 {
 	int result;
 
-#if defined(WIN32) || defined(_WINDOWS)
-	if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0) {
-		fprintf(stderr, "could not initialize winsock\n");
-		exit(1);
-	}
-	wsaInit++;
-#endif
-
+	InitWinsock();
 #ifdef HAVE_SETLOCALE
 	setlocale(LC_ALL, "");
 #endif
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	gIsTTY = 1;
 	gIsTTYr = 1;
+#elif defined(__CYGWIN__)
+	gIsTTY = (isatty(2) != 0) ? 1 : 0;
+	gIsTTYr = (isatty(0) != 0) ? 1 : 0;
 #else
 	gIsTTY = ((isatty(2) != 0) && (getppid() > 1)) ? 1 : 0;
 	gIsTTYr = ((isatty(0) != 0) && (getppid() > 1)) ? 1 : 0;
@@ -274,13 +280,19 @@ PreInit(void)
 #ifdef SIGPOLL
 	(void) NcSignal(SIGPOLL, (FTPSigProc) SIG_IGN);
 #endif
+	gXBufSize = kDefaultFTPBufSize;
+	gXBuf = malloc(gXBufSize);
+	if (gXBuf == NULL) {
+		perror("malloc");
+		exit(1);
+	}
 	InitUserInfo();
 	result = FTPInitLibrary(&gLib);
 	if (result < 0) {
 		(void) fprintf(stderr, "ncftp: init library error %d (%s).\n", result, FTPStrError(result));
 		exit(1);
 	}
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	srand((unsigned int) (GetTickCount() & 0x7FFF));
 #else
 	srand((unsigned int) getpid());
@@ -295,7 +307,6 @@ PreInit(void)
 	InitCommandList();
 	InitLs();
 	TruncBatchLog();
-	GetoptReset();
 	GetScreenColumns();
 }	/* PreInit */
 
@@ -310,6 +321,8 @@ static void
 PostInit(void)
 {
 	PostInitPrefs();
+	if (gGetOurHostNameResult == 100)	/* avoid doing again */
+		gGetOurHostNameResult = GetOurHostName(gOurHostName, sizeof(gOurHostName));
 	OpenTrace();
 	InitTermcap();
 	InitReadline();
@@ -327,7 +340,7 @@ PostInit(void)
 		gFirewallPort
 	);
 	Trace(0, "FwExceptions: %s\n", gFirewallExceptionList);
-	if (strchr(gLib.ourHostName, '.') == NULL) {
+	if (strchr(gOurHostName, '.') == NULL) {
 		Trace(0, "NOTE:  Your domain name could not be detected.\n");
 		if (gConn.firewallType != kFirewallNotInUse) {
 			Trace(0, "       Make sure you manually add your domain name to firewall-exception-list.\n");
@@ -342,11 +355,11 @@ PostInit(void)
 static void
 Plug(void)
 {
-#if defined(WIN32) || defined(_WINDOWS)
+#if defined(WIN32) || defined(_WINDOWS) || defined(__CYGWIN__)
 	/* NcFTPd hasn't been ported to Windows. */
 #else
-	if ((gNumProgramRuns % 7) == 2) {
-		(void) printf("\n\n\n\tThank you for using NcFTP.\n\tAsk your system administrator to try NcFTPd Server!\n\thttp://www.ncftp.com\n\n\n\n");
+	if ((gDoNotDisplayAds == 0) && ((gNumProgramRuns % 7) == 2)) {
+		(void) printf("\n\n\n\tThank you for using NcFTP Client.\n\tAsk your system administrator to try NcFTPd Server!\n\thttp://www.ncftpd.com\n\n\n\n");
 	}
 #endif
 }	/* Plug */
@@ -360,34 +373,42 @@ PostShell(void)
 {
 	SetXtermTitle("RESTORE");
 	CloseHost();
+	FlushLsCache();
 	DisposeReadline();
 	CloseTrace();
 	SavePrefs();
+	DisposeBookmarkTable();
 	EndLog();
 	Plug();
-	DisposeWinsock(0);
+	if (gXBuf != NULL)
+		free(gXBuf);
+	DisposeWinsock();
 }	/* PostShell */
 
 
 
 
-int
-main(int argc, const char **const argv)
+main_void_return_t
+main(int argc, char **const argv)
 {
 	int c;
 	int n;
+	GetoptInfo opt;
 
 	PreInit();
-	while ((c = Getopt(argc, argv, "P:u:p:J:rd:g:FVLD")) > 0) switch(c) {
+	GetoptReset(&opt);
+	while ((c = Getopt(&opt, argc, argv, "P:u:p:j:rd:eg:o:FVLD")) > 0) switch(c) {
 		case 'P':
 		case 'u':
 		case 'p':
-		case 'J':
+		case 'j':
 			gStartupUrlParameterGiven = 1;
 			break;
 		case 'r':
 		case 'g':
 		case 'd':
+		case 'e':
+		case 'o':
 		case 'V':
 		case 'L':
 		case 'D':
@@ -397,9 +418,9 @@ main(int argc, const char **const argv)
 			Usage();
 	}
 
-	if (gOptInd < argc) {
+	if (opt.ind < argc) {
 		LoadFirewallPrefs(0);
-		SetStartupURL(argv[gOptInd]);
+		SetStartupURL(argv[opt.ind]);
 	} else if (gStartupUrlParameterGiven != 0) {
 		/* One of the flags they specified
 		 * requires a URL or hostname to
@@ -408,40 +429,54 @@ main(int argc, const char **const argv)
 		Usage();
 	}
 
-	GetoptReset();
 	/* Allow command-line parameters to override
 	 * bookmark settings.
 	 */
-
-	while ((c = Getopt(argc, argv, "P:u:p:J:rd:g:FVLD")) > 0) switch(c) {
+	GetoptReset(&opt);
+	while ((c = Getopt(&opt, argc, argv, "P:u:p:j:rd:eg:o:FVLD")) > 0) switch(c) {
 		case 'P':
-			gConn.port = atoi(gOptArg);	
+			gConn.port = atoi(opt.arg);	
 			break;
 		case 'u':
-			(void) STRNCPY(gConn.user, gOptArg);
+			(void) STRNCPY(gConn.user, opt.arg);
+			memset(opt.arg, '*', strlen(opt.arg));
 			break;
 		case 'p':
-			(void) STRNCPY(gConn.pass, gOptArg);	/* Don't recommend doing this! */
+			(void) STRNCPY(gConn.pass, opt.arg);	/* Don't recommend doing this! */
+			memset(opt.arg, '*', strlen(opt.arg));
 			break;
-		case 'J':
-			(void) STRNCPY(gConn.acct, gOptArg);
+		case 'j':
+			(void) STRNCPY(gConn.acct, opt.arg);
+			memset(opt.arg, '*', strlen(opt.arg));
+			break;
+		case 'e':
+			gGetOurHostNameResult = GetOurHostName(gOurHostName, sizeof(gOurHostName));
+			printf("%s\n", gOurHostName);
+			exit(0);
 			break;
 		case 'r':
 			/* redial is always on */
 			break;
 		case 'g':
-			gConn.maxDials = atoi(gOptArg);
+			gConn.maxDials = atoi(opt.arg);
 			break;
 		case 'd':
-			n = atoi(gOptArg);
+			n = atoi(opt.arg);
 			if (n >= 10)
 				gConn.redialDelay = n;
 			break;
+		case 'o':
+			gConn.manualOverrideFeatures = opt.arg;
+			break;
 		case 'F':
 			DumpFirewallPrefsTemplate();
 			exit(0);
+			/*NOTREACHED*/
+			break;
 		case 'V':
+			/*FALLTHROUGH*/
 		case 'L':
+			/*FALLTHROUGH*/
 		case 'D':
 			/* silently ignore these legacy options */
 			break;
@@ -449,6 +484,9 @@ main(int argc, const char **const argv)
 			Usage();
 	}
 
+#if (defined(SOCKS)) && (SOCKS >= 5)
+	SOCKSinit(argv[0]);
+#endif	/* SOCKS */
 	PostInit();
 	OpenURL();
 	CommandShell();
